Package: supersigs
Title: Supervised mutational signatures
Version: 1.19.0
Date: 2021-12-02
Authors@R: c(person(given = "Albert", 
                    family = "Kuo",
                    role = c("aut", "cre"),
                    email = "albertkuo@jhu.edu", 
                    comment = c("ORCID" = "0000-0001-5155-0748")),
             person(given = "Yifan", 
                    family = "Zhang",
                    role = c("aut")),
             person(given = "Bahman", 
                    family = "Afsari",
                    role = c("aut")),
             person(given = "Cristian", 
                    family = "Tomasetti",
                    role = c("aut"))) 
Depends: R (>= 4.1)
Imports: assertthat, caret, dplyr, tidyr, rsample, methods, rlang,
        utils, Biostrings, stats, SummarizedExperiment
Suggests: BSgenome.Hsapiens.UCSC.hg19, BSgenome.Hsapiens.UCSC.hg38,
        knitr, rmarkdown, ggplot2, testthat, VariantAnnotation
Description: Generate SuperSigs (supervised mutational signatures) from single nucleotide variants in the cancer genome. Functions included in the package allow the user to learn supervised mutational signatures from their data and apply them to new data. The methodology is based on the one described in Afsari (2021, ELife).
biocViews: FeatureExtraction, Classification, Regression, Sequencing,
        WholeGenome, SomaticMutation
BugReports: https://github.com/TomasettiLab/supersigs/issues
URL: https://tomasettilab.github.io/supersigs/
License: GPL-3
Encoding: UTF-8
LazyData: true
LazyDataCompression: gzip
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/supersigs
git_branch: devel
git_last_commit: 281d3c5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:31:34 UTC; biocbuild
Author: Albert Kuo [aut, cre] (ORCID: <https://orcid.org/0000-0001-5155-0748>),
  Yifan Zhang [aut],
  Bahman Afsari [aut],
  Cristian Tomasetti [aut]
Maintainer: Albert Kuo <albertkuo@jhu.edu>

# Generated by roxygen2: do not edit by hand

export(get_signature)
export(make_matrix)
export(partial_signature)
export(predict_signature)
export(process_vcf)
export(simplify_signature)
import(assertthat)
import(caret)
import(dplyr)
import(rsample)
import(tidyr)
import(utils)
importFrom(Biostrings,getSeq)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(SummarizedExperiment,seqinfo)
importFrom(methods,as)
importFrom(methods,new)
importFrom(rlang,.data)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,end)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,pbinom)
importFrom(stats,predict)
importFrom(stats,setNames)
importFrom(stats,start)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# supersigs

<!-- badges: start -->

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-blue.svg)](https://www.tidyverse.org/lifecycle/#experimental)
<!-- [![CRAN Status](https://www.r-pkg.org/badges/version/pkgdown)](https://cran.r-project.org/package=pkgdown) -->
<!-- [![R build status](https://github.com/r-lib/pkgdown/workflows/R-CMD-check/badge.svg)](https://github.com/r-lib/supersigs/actions) -->
<!-- [![Codecov test coverage](https://codecov.io/gh/r-lib/pkgdown/branch/master/graph/badge.svg)](https://codecov.io/gh/r-lib/supersigs?branch=master) -->
<!-- badges: end -->

`supersigs` is a companion R package to a method proposed by *Afsari, et
al. (2021, ELife)* to generate mutational signatures from single
nucleotide variants in the cancer genome. **Note: Package is under
active development.**

More details on the statistical method can be found in this paper:

-   Afsari, B., Kuo, A., Zhang, Y., Li, L., Lahouel, K., Danilova, L.,
    Favorov, A., Rosenquist, T. A., Grollman, A. P., Kinzler, K. W.,
    Cope, L., Vogelstein, B., & Tomasetti, C. (2021). Supervised
    mutational signatures for obesity and other tissue-specific
    etiological factors in cancer. ELife, 10.
    [https://doi.org/10.7554/elife.61082](https://doi.org/10.7554/eLife.61082)

## Installation

``` r
# Install package from Bioconductor
if(!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("supersigs")
```

You can also install the development version of supersigs from github
using the `install_github()` function from the `devtools` package.

``` r
# Install development version from GitHub
devtools::install_github("TomasettiLab/supersigs")
```

## Data format

At a minimum, the data you will need are the age and mutations for each
patient. An example is provided below. (Note that you will need to
process the data before running the core functions, see
`vignette("supersigs")` for details.)

    #>   sample_id age chromosome  position ref alt
    #> 1         1  50       chr1  94447621   G   C
    #> 2         1  50       chr2 202005395   A   C
    #> 3         1  50       chr7  20784978   T   A
    #> 4         1  50       chr7  87179255   C   G
    #> 5         1  50      chr19   1059712   G   T
    #> 6         2  55       chr1  76226977   T   C

## Core functions

In brief, the `supersigs` package contains three core functions:
`get_signature`, `predict_signature`, and `partial_signature`.

`get_signature` trains a supervised signature for a given factor
(e.g. smoking).

``` r
supersig <- get_signature(data = data, factor = "smoking", wgs = F)
```

`predict_signature` uses the trained supervised signature to obtain
predicted probabilities (e.g. probability of smoker) on a new dataset.

``` r
pred <- predict_signature(object = supersig, newdata = data, factor = "smoking")
```

`partial_signature` removes the contribution of a trained signature from
the dataset.

``` r
data <- partial_signature(data = data, object = supersig)
```

## Tutorial

To follow a tutorial on how to use the package, see
`vignette("supersigs")` (or type `vignette("supersigs")` in R).

# condense_mutations.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Dec 10, 2019
#
# Helper function for GenerateMinSigmaAlgebra.R

#' Given a set of level 3 features, simplify and condense into level 1 and 
#' level 2 mutations
#' 
#' @param features a vector of level 3 features
#'
#' @return \code{condense_mutations} returns a vector of condensed features
#' 
#' @noRd
#' 
condense_mutations = function(features){
    out <- vector('character')
    for(i in seq_along(muts_children_level3)){
        v <- muts_children_level3[[i]]
        if(all(v %in% features)){
            out <- c(out, names(muts_children_level3)[i])
            features <- setdiff(features, v)
        }
    }
    return(out)
}

x <- "[C>A]T"
x_div <- c("A[C>A]T", "C[C>A]T", "G[C>A]T", "T[C>A]T")
y <- "A[C>A]T"

test_that("condense_mutations works", {
    expect_equal(condense_mutations(x_div), x)
    expect_equal(y, y)
})

test_that("convert_to_level3 works", {
    expect_equal(convert_to_level3(x), x_div)
    expect_equal(convert_to_level3(y), y)
})

# partial_signature.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Feb 25, 2021
#
# (Export) Function to remove signature contribution from data

# library(dplyr)

#' Function to remove the contribution of a SuperSig
#' 
#' Remove the contribution of a SuperSig from the data and return the data.
#' 
#' @param data a data frame of mutations containing columns for
#' \code{sample_id}, \code{age}, \code{IndVar}, and the 96 trinucleotide
#' mutations (see vignette for details)
#' @param object an object of class \code{SuperSig}
#' 
#' @import dplyr
#' 
#' @export
#' 
#' @return \code{predict_signature} returns the original data frame with 
#' the contribution of a supervised signature removed
#' 
#' @examples
#' 
#' head(example_dt) # use example data from package
#' input_dt <- make_matrix(example_dt) # convert to correct format
#' input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
#' supersig <- get_signature(data = input_dt, factor = "Age") # get SuperSig
#' partial_signature(data = input_dt, object = supersig)
#' 
partial_signature <- function(data, object){
    # Check column names of data
    if(is.na(match("SAMPLE_ID",toupper(colnames(data))))){
        stop('Input data frame missing sample_id column.')
    } else {
        colnames(data)[match("SAMPLE_ID",toupper(colnames(data)))] = "sample_id"
    }
    if(is.na(match("AGE",toupper(colnames(data))))){
        stop('Input data frame missing AGE column.')
    } else {
        colnames(data)[match("AGE",toupper(colnames(data)))] = "AGE"
    }
    if(is.na(match("INDVAR",toupper(colnames(data))))){
        stop('Input data frame missing IndVar column.')
    } else {
        colnames(data)[match("INDVAR",toupper(colnames(data)))] = "IndVar"
    }
    
    # Extract slots from object
    features <- Features(object)
    
    # Subtract contributions (rates*age)
    for(i in seq_along(features)){
        trinucleotide_features <- unname(features[[i]])
        diff <- Signature(object)[[i]]
        data <- data %>%
            mutate_at(vars(all_of(trinucleotide_features)), ~.- diff*AGE) %>%
            mutate_at(vars(all_of(trinucleotide_features)), 
                      ~ifelse(. < 0, 0, .))
    }
    
    return(data)
}

# supersig_classifier.R
# -----------------------------------------------------------------------------
# Author: Bahman Afsari, Albert Kuo
# Date last modified: Feb 25, 2021
#
# Function for classification logistic regression

# Run logistic regression
run_logit <- function(dt, factor, out, keep_classifier, features_selected,
                      select_n, train_ind, test_ind, test_indvar){
    z <- dt %>%
        select(c(features_selected[seq_len(select_n["Logit"])], "IndVar")) 
    x <- z[train_ind, ]
    newdata <- z[test_ind, ]
    out$auc <- c(out$auc, Logit = NA)
    logit_prediction <- NA
    
    try({
        logit_classifier <- glm(formula = IndVar ~ ., data = x,
                                family = binomial())
        logit_prediction <- predict(logit_classifier, 
                                    newdata = newdata, 
                                    type = "response")
        features_to_summ <- features_selected[seq_len(select_n["Logit"])]
        # Calculate average rates
        if(factor == "AGE"){
            dt_train <- dt %>% slice(train_ind)
            mean_age <- mean(dt$AGE, na.rm = TRUE)
            mean_diffs <- dt %>%
                summarize_at(.vars = features_to_summ,
                             .funs = ~(mean(.)/mean_age))
        } else {
            # Calculate empirical mean differences in rates
            grouped_rates <- dt %>% slice(train_ind) %>% group_by(.data$IndVar)
            grouped_rates <- grouped_rates %>%
                summarize_at(.vars = features_to_summ,
                             .funs = ~(mean(./AGE, na.rm = TRUE)))
            unexposed_rates <- grouped_rates %>% 
                filter(.data$IndVar == FALSE) %>% 
                select(-.data$IndVar)
            exposed_rates <- grouped_rates %>% 
                filter(.data$IndVar == TRUE) %>% 
                select(-.data$IndVar)
            mean_diffs <- exposed_rates - unexposed_rates
        }
        # Save signature for apparent
        if(identical(test_ind, train_ind)){
            names(mean_diffs) <- features_selected[seq_len(select_n["Logit"])]
            
            out$signature <- list(mean_diffs = mean_diffs,
                                  select_n = select_n)
        }
        out$auc["Logit"] <- my_auc(test_indvar, logit_prediction)
        if(keep_classifier) 
            out$classifier$Logit <- logit_classifier
    })
    return(out)
}

#' Classification of exposure using signatures
#' 
#' Calculate the AUC using signatures and logistic regression
#' 
#' @param dt a transformed data frame from FeatureSelection
#' @param test_ind an optional vector of indices for the test data
#' @param factor the factor/exposure (e.g. "age", "smoking")
#' @param keep_classifier an optional logical value indicating whether to save
#' the classifier model (default is \code{FALSE})
#' @param features_selected a vector of candidate features ranked by AUC
#' @param select_n the number of top features to retain for each method
#' 
#' @import dplyr
#' @import rsample
#' @importFrom rlang .data
#'
#' @return \code{supersig_classifier} returns a list of several elements:
#' \itemize{
#' \item \code{auc} is a vector of AUCs for each classifier
#' \item \code{signature} is a vector of mean differences or rates
#' (created only when \code{classifier} includes "Logit")
#' \item \code{classifier} is a list of saved models for each input classifier
#' (created only when \code{keep_classifier} is \code{TRUE})
#' }
#' 
#' @noRd
#' 
supersig_classifier <- function(dt, 
                                test_ind = NULL,
                                factor,
                                keep_classifier = FALSE,
                                features_selected,
                                select_n){
        # Split training and test set
        if(is.null(test_ind)){
                train_ind <- seq_len(nrow(dt))
                test_ind <- train_ind
        } else {
                train_ind <- setdiff(seq_len(nrow(dt)), test_ind)
        }
        
        train <- dt[train_ind, ]
        # Initialize output
        test_indvar <- dt[test_ind, "IndVar"]
        out <- list(auc = c(), signature = list(), classifier = list())

        # Transform data
        if(factor != "AGE"){
                dt <- dt %>%
                        mutate_at(.vars = features_selected,
                                  .funs = funs(./.data$AGE))
                
                # Remove observations missing age
                missing_ind <- which(is.na(dt$AGE))
                train_ind <- setdiff(train_ind, missing_ind)
                test_ind <- setdiff(test_ind, missing_ind)
                train <- dt[train_ind, ]
                test_indvar <- dt[test_ind, "IndVar"]
        } 
        
        # Classification
        # If no test data or all training data has the same IndVar
        if(length(test_ind) == 0 || length(unique(train$IndVar)) == 1){
                warning("No valid test data")
                out$auc <- c(NA)
        } else {
            out <- run_logit(dt, factor, out, 
                             keep_classifier, features_selected, 
                             select_n, train_ind, test_ind, test_indvar)
        }
        
        return(out)
}

# make_matrix.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Feb 22, 2021
#
# (Export) Function to transform data frame of mutations into correct format

# Wrapper for getSeq given specified genome (helper function)
getseq_wrapper <- function(dt, genome = "hg19"){
  if(genome == "hg19"){
    if (!requireNamespace("BSgenome.Hsapiens.UCSC.hg19", quietly = TRUE)){
      stop("Package \"BSgenome.Hsapiens.UCSC.hg19\" 
                 needed for this function to work. Please install it.",
           call. = FALSE)
    }
    dt_ranges <- as(dt %>% 
                      select(.data$chromosome, .data$start, .data$end), 
                    "GRanges")
    aligned_dna <- 
      getSeq(BSgenome.Hsapiens.UCSC.hg19::BSgenome.Hsapiens.UCSC.hg19, 
             dt_ranges)
  } else if(genome == "hg38"){
    if (!requireNamespace("BSgenome.Hsapiens.UCSC.hg38", quietly = TRUE)){
      stop("Package \"BSgenome.Hsapiens.UCSC.hg38\" 
                 needed for this function to work. Please install it.",
           call. = FALSE)
    }
    dt_ranges <- as(dt %>% select(.data$chromosome, .data$start, .data$end), 
                    "GRanges")
    aligned_dna <- 
      getSeq(BSgenome.Hsapiens.UCSC.hg38::BSgenome.Hsapiens.UCSC.hg38, 
             dt_ranges)
  } else {
    stop("Invalid genome specified")
  }
  return(aligned_dna)
}

#' Function to transform mutations into "matrix" format
#' 
#' Transform a data frame of mutations in long format into 
#' a data frame of trinucleotide mutations with flanking bases
#' in a wide matrix format.
#' 
#' @param data a data frame of mutations in VCF format 
#' (see vignette for details)
#' @param genome the reference genome used ("hg19" or "hg38")
#' 
#' @import dplyr
#' @import tidyr
#' @importFrom Biostrings getSeq
#' @importFrom rlang .data
#' 
#' @export
#' 
#' @return \code{make_matrix} returns a data frame of mutations,
#' one row per sample
#' 
#' @examples
#' 
#' head(example_dt) # use example data from package
#' input_dt <- make_matrix(example_dt) # convert to correct format
#' head(input_dt)
#' 
make_matrix <- function(data, genome = "hg19"){
    data <- data %>%
        select(.data$sample_id, .data$age, .data$chromosome, .data$position, 
               .data$ref, .data$alt) %>%
        mutate(start = .data$position - 1,
               end = .data$position + 1)
    
    aligned_dna <- getseq_wrapper(data, genome)
    
    # Create mutations with surrounding base pairs
    data <- data %>%
        mutate(aligned = as.character(aligned_dna),
               mutation = paste0(substr(.data$aligned, 1, 1), "[", 
                                 .data$ref, ">", .data$alt, "]",
                                 substr(.data$aligned, 3, 3)),
               mutation_std = vapply(.data$mutation,
                                     function(x){transform_muts_vec[[x]]},
                                     FUN.VALUE = character(1)) %>% unname())
    
    # Count mutations for each patient
    dt_counts <- data %>%
        group_by(.data$sample_id, .data$age, .data$mutation_std) %>%
        summarize(mut_count = n()) %>%
        ungroup() %>%
        pivot_wider(names_from = .data$mutation_std, 
                    values_from = .data$mut_count) %>%
        mutate_all(~replace(., is.na(.), 0))
    
    # Add any fundamental mutations that are missing
    for(mut in transform_muts_vec){
        if(!(mut %in% names(dt_counts))){
            dt_counts <- dt_counts %>%
                mutate(!!mut := 0)
        }
    }
    
    return(dt_counts)
}

# process_vcf.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Feb 11, 2021
#
# (Export) Function to transform VCF object into correct format

# library(dplyr)
# library(tidyr)

#' Function to transform VCF object into "matrix" format
#' 
#' Transform a VCF object into 
#' a data frame of trinucleotide mutations with flanking bases
#' in a wide matrix format. The function assumes that the VCF object
#' contains only one sample and that each row in rowRanges 
#' represents an observed mutation in the sample.
#' 
#' @param vcf a VCF object (from \code{VariantAnnotation} package)
#' 
#' @import dplyr
#' @importFrom SummarizedExperiment colData rowRanges seqinfo
#' @importFrom rlang .data
#' 
#' @export
#' 
#' @return \code{process_vcf} returns a data frame of mutations, 
#' one row per mutation
#' 
#' @examples
#' 
#' # Use example vcf from VariantAnnotation
#' suppressPackageStartupMessages({library(VariantAnnotation)})
#' fl <- system.file("extdata", "chr22.vcf.gz", package="VariantAnnotation")
#' vcf <- VariantAnnotation::readVcf(fl, "hg19") 
#' 
#' # Subset to first sample
#' vcf <- vcf[, 1]
#' # Subset to row positions with homozygous or heterozygous alt
#' positions <- geno(vcf)$GT != "0|0" 
#' vcf <- vcf[positions[, 1],]
#' colData(vcf)$age <- 50        # Add patient age to colData (optional)
#' 
#' # Run function
#' dt <- process_vcf(vcf)
#' head(dt)
#' 
process_vcf <- function(vcf){
    genome <- unname(genome(SummarizedExperiment::seqinfo(vcf)))
    n_samples <- nrow(colData(vcf))
    assert_that(n_samples == 1, msg = "Number of samples not equal to 1 in VCF")
    
    sample_ids <- rownames(SummarizedExperiment::colData(vcf))
    if("age" %in% colnames(SummarizedExperiment::colData(vcf))){
        ages <- colData(vcf)$age
    } else {
        ages <- rep(NA, n = n_samples)
    }
    
    dt <- data.frame(SummarizedExperiment::rowRanges(vcf)) %>%
            dplyr::mutate(chromosome = paste0("chr", .data$seqnames)) %>%
            dplyr::rename(position = .data$start) %>%
            dplyr::mutate(ref = vapply(.data$REF, function(x)
            {as.character(x)[[1]][[1]]},
            FUN.VALUE = character(1)),
            alt = vapply(.data$ALT, function(x)
            {as.character(x)[[1]][[1]]},
            FUN.VALUE = character(1)),
            sample_id = sample_ids[1],
            age = ages[1]) %>%
            dplyr::filter(nchar(.data$ref) == 1 & nchar(.data$alt) == 1) %>%
            dplyr::select(.data$sample_id, .data$age, .data$chromosome, 
                          .data$position, .data$ref, .data$alt)
    
    return(dt)
}

# SuperSig-class.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Feb 25, 2021
#' An S4 class for SuperSig
#'
#' @slot Signature data frame of features and their 
#' difference in mean rates between exposed and unexposed
#' (or the average rate if the factor is "age")
#' @slot Features list of features that comprise the signature and 
#' their representation in terms of the fundamental (trinucleotide) mutations
#' @slot AUC length-one numeric vector of the apparent AUC 
#' (i.e. not cross-validated)
#' @slot Model list of a glm class for trained logistic regression model
#' 
SuperSig <- setClass("SuperSig",
                     slots = list(Signature = "data.frame",
                                  Features = "list",
                                  AUC = "numeric",
                                  Model = "list"),
                     contains = "glm")


# Display ---
# class method for show() generic
setMethod("show",
                    "SuperSig",
                    function(object) {
                        cat("Signature:\n")
                        print(object@Signature)
                        cat("Features:\n")
                        print(object@Features)
                        cat("Model:\n")
                        print(object@Model)
                    }
)

# class method for print
print.SuperSig = function(object){
    object
}

# Accessors ---
setGeneric("Signature", function(x) standardGeneric("Signature"))
setMethod("Signature", "SuperSig", function(x) x@Signature)

setGeneric("Features", function(x) standardGeneric("Features"))
setMethod("Features", "SuperSig", function(x) x@Features)

setGeneric("AUC", function(x) standardGeneric("AUC"))
setMethod("AUC", "SuperSig", function(x) x@AUC)

setGeneric("Model", function(x) standardGeneric("Model"))
setMethod("Model", "SuperSig", function(x) x@Model)

# predict_signature.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Dec 23, 2020
#
# (Export) Function to apply SuperSigs model on new dataset

#' Function to predict using SuperSig object
#' 
#' Using a generated SuperSig, predict on a new dataset and return 
#' predicted probabilities for each observation. 
#' 
#' @param object an object of class \code{SuperSig}
#' @param newdata a data frame of mutations containing columns for
#' \code{sample_id}, \code{age}, \code{IndVar}, and the 96 trinucleotide
#' mutations (see vignette for details)
#' @param factor the factor/exposure (e.g. "age", "smoking")
#' 
#' @import dplyr
#' @importFrom methods as new
#' @importFrom stats binomial coef end glm median pbinom predict setNames start
#' 
#' @export
#' 
#' @return \code{predict_signature} returns the original data frame with 
#' additional columns for the feature counts and classification score
#' 
#' @examples
#' 
#' head(example_dt) # use example data from package
#' input_dt <- make_matrix(example_dt) # convert to correct format
#' input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
#' out <- get_signature(data = input_dt, factor = "Age") # get SuperSig
#' newdata <- predict_signature(out, newdata = input_dt, factor = "age")
#' suppressPackageStartupMessages({library(dplyr)})
#' head(newdata %>% select(score))
#' 
predict_signature <- function(object, newdata, factor){
    # Capitalize factor string
    factor <- toupper(factor)
    
    # Check column names of dt
    if(is.na(match("AGE",toupper(colnames(newdata))))){
      stop('Input data frame missing AGE column.')
    } else {
      colnames(newdata)[match("AGE",toupper(colnames(newdata)))] <- "AGE"
    }
    
    # Extract slots from object
    model <- Model(object)$Logit
    features <- Features(object)
    
    # Add counts for features that are used in the model
    for(j in seq_along(features)){
        feature <- names(features)[j]
        feature_count <- newdata %>%
            select(features[[feature]]) %>%
            rowSums()
        newdata <- newdata %>%
            mutate(!!feature := feature_count)
    }
    
    # Use rates for non-age factors
    if(factor != "AGE"){
        newdata <- newdata %>%
            mutate_at(names(features), ~(./AGE))
    }
    
    # Predict using logistic regression
    scores <- predict(model, 
                      newdata = newdata, 
                      type = "response")
    
    newdata <- newdata %>%
        mutate(score = scores)
    
    return(newdata)
}

# feature_selection.R
# -----------------------------------------------------------------------------
# Author: Bahman Afsari, Albert Kuo
# Date last modified: Feb 22, 2021
#
# Function for selecting features

# Add up counts for every mutation (helper function)
add_counts <- function(dt){
    dt <- dt %>%
        transmute_(.dots = muts_formula) %>%
        mutate(TOTAL_MUTATIONS = 
                         select(., c("C>A", "C>G", "C>T", 
                                     "T>A", "T>C", "T>G")) %>%
                         rowSums())
    
    return(dt)
}

# Apply context matters to every inner partition (helper function)
feature_engineering <- function(train, factor, inner_partitions, n_fold, wgs){
    message("Begin feature engineering...")
    new_partition_ls <- vector(length = length(inner_partitions), 
                               mode = "list")
    for(ij in seq_along(inner_partitions)){
        i <- (ij-1) %% n_fold + 1
        j <- floor((ij-1)/n_fold) + 1
        
        test_inner_ind <- inner_partitions[[i, j]]
        train_inner_ind <- setdiff(seq_len(nrow(train)), test_inner_ind)
        train_inner <- train[train_inner_ind, ]

        # Separate into exposed (train_1) and unexposed (train_0)
        if(factor != "AGE"){ 
            train_0 <- train_inner %>% filter(.data$IndVar == 0)
            train_1 <- train_inner %>% filter(.data$IndVar == 1)
        } else {
            train_0 <- train_inner 
            train_1 <- train_0
        }
        
        # Add up counts for every mutation
        train_0 <- add_counts(train_0)
        train_1 <- add_counts(train_1)

        # Test for significant features
        if(nrow(train_0) == 0 || nrow(train_1) == 0) next
        features_context_0 <- context_matters(train_0, wgs = wgs)
        features_context_1 <- context_matters(train_1, wgs = wgs)
        
        input_ls <- list(var0 = features_context_0, var1 = features_context_1)
        features_gmsa <- generate_min_sigma_algebra(input_ls)
        new_partition_ls[[ij]] <- features_gmsa
    }
    
    # Repartition features
    new_partition_combined <- Reduce(function(a, b){
        generate_min_sigma_algebra(list(var0 = a, var1 = b),
                                   partitioned_features = TRUE)},
        new_partition_ls)
    
    return(new_partition_combined)
}

# Rank features by AUC (helper function)
rank_tri_features <- function(dt_new, inner_partitions,
                              new_partition, factor){
    rank_tri_ls <- vector(length = length(inner_partitions), mode = "list")
    for(ij in seq_along(inner_partitions)){
        feature_names <- names(new_partition)
        auc_mat <- all_my_auc(dt_new %>% select(c(feature_names, "IndVar")), 
                                                    IndVar = "IndVar")
        if(factor == "AGE"){
            auc_medians <- auc_mat
        } else {
            auc_medians <- ifelse(auc_mat > 0.5, auc_mat, 1 - auc_mat)
        }
        rank_mutations <- tibble(mutation = feature_names,
                                 auc = auc_medians) %>%
            arrange(desc(.data$auc))
        
        # Calculate rank of trinucleotide equivalents
        feature_to_trinuc <- tibble(feature = rep(feature_names,
                                                  vapply(new_partition, 
                                                                 length,
                                                                 numeric(1))),
                                    trinucleotide = unlist(new_partition))
        rank_trinucleotides <- rank_mutations %>%
            left_join(., feature_to_trinuc,
                                by = c("mutation" = "feature")) %>%
            mutate(rank = seq_len(n())) %>%
            group_by(.data$mutation) %>%
            mutate(mean_rank = mean(.data$rank)) %>%
            ungroup()
        
        # Save trinucleotide rankings
        rank_tri_ls[[ij]] <- rank_trinucleotides %>%
            select(.data$trinucleotide, .data$mean_rank) %>%
            arrange(.data$trinucleotide) %>%
            dplyr::rename(!!paste0("rank_", ij) := .data$mean_rank,
                          !!paste0("trinucleotide_", ij) := .data$trinucleotide)
    }
    
    return(rank_tri_ls)
}

# Get repartition of features (helper function)
repartition_with_rank <- function(rank_tri_ls){
    # Take median of rankings for trinucleotides over inner folds
    rank_tri <- bind_cols(rank_tri_ls)
    rank_tri <- rank_tri %>%
        mutate(rank = apply(select(., starts_with("rank")), 1, median)) %>%
        select(.data$trinucleotide_1, .data$rank) %>%
        dplyr::rename(trinucleotide = .data$trinucleotide_1) %>%
        arrange(.data$rank)
    
    # Create new partition of features, grouping trinucleotides of same rank
    # into one feature
    unique_ranks <- unique(rank_tri$rank)
    features_new <- lapply(unique_ranks, function(r) rank_tri %>%
                             filter(.data$rank == r) %>%
                             pull(.data$trinucleotide)) 
    names(features_new) <- paste0("X", seq_along(features_new))
    new_partition <- features_new
    features_gmsa <- list(new_partition = features_new)
    features_selected <- names(new_partition)
    
    return(list(features_gmsa, new_partition, features_selected))
}

# Get n star (helper function)
n_star_features <- function(train, factor, inner_partitions,
                            n_fold, features_selected){
    n_star_ls <- vector(length = length(inner_partitions), mode = "list")
    
    n <- length(features_selected)
    cv_msg <- paste("Begin cross-validated selection over", n, 
                          "features and", 
                          length(inner_partitions), "inner folds...")
    message(cv_msg)
    
    for(ij in seq_along(inner_partitions)){
        partition_msg <- paste("...testing inner fold", ij)
        message(partition_msg)
        i <- (ij-1) %% n_fold + 1
        j <- floor((ij-1)/n_fold) + 1
        
        test_inner_ind <- inner_partitions[[i, j]]
        train_inner_ind <- setdiff(seq_len(nrow(train)), test_inner_ind)
        
        # Find n*
        auc_ls <- list()
        methods <- c("Logit")
        
        for(k in seq_len(n)){
            auc_ls[[k]] <- supersig_classifier(dt = train, 
                                               test_ind = test_inner_ind,
                                               factor,
                                               keep_classifier = FALSE,
                                               features_selected,
                                               select_n = c("Logit" = k))$auc
        }
        n_star <- which.max(vapply(auc_ls, function(x) x["Logit"],
                                   FUN.VALUE = numeric(1)))
        n_star <- ifelse(identical(n_star, integer(0)), NA, n_star)
        
        # Save n_star for each method
        n_star_ls[[ij]] <- tibble(!!paste0("methods_", ij) := methods,
                                  !!paste0("n_star_", ij) := n_star)
    }
    
    return(n_star_ls)
}

# Get median n star (helper function)
median_n_star <- function(n_star_ls){
    n_star <- bind_cols(n_star_ls)
    n_star <- n_star %>%
        mutate(n_star = apply(select(., starts_with("n_star")), 1, 
                              median, na.rm = TRUE),
               n_star = round(.data$n_star)) %>%
        select(.data$methods_1, .data$n_star) %>%
        dplyr::rename(methods = .data$methods_1)
    
    select_n <- n_star$n_star
    names(select_n) <- n_star$methods
    
    return(select_n)
}

# Adjust select_n by removing features that are not > 0.6 AUC
select_n_adjust <- function(train, features_selected, select_n){
    auc_mat <- all_my_auc(train %>% select(c(features_selected, "IndVar")), 
                          IndVar = "IndVar")
    max_n <- sum(auc_mat > 0.6)
    select_n <- vapply(select_n, function(x) min(max_n, x),
                       FUN.VALUE = numeric(1))
}

#' Function for feature selection
#' 
#' Perform feature selection given a dataset of mutations by calling 
#' a series of other functions to find significant and predictive features
#' 
#' @param dt a data frame of mutations
#' @param factor the factor/exposure (e.g. "age", "smoking")
#' #' @param wgs logical value indicating whether sequencing data is 
#' whole-genome (wgs = \code{TRUE}) or whole-exome (wgs = \code{FALSE}). 
#' 
#' @import dplyr
#' @import assertthat
#' @import caret
#' @importFrom rlang .data
#' 
#' @return \code{feature_selection} returns a list of several elements:
#' \itemize{
#' \item \code{features_context_0} is a vector of survival mutations
#' for the unexposed group
#' \item \code{features_context_1} is a vector of survival mutations
#' for the exposed group
#' \item \code{features_selected} is a vector of candidate 
#' features ranked by AUC
#' \item \code{new_partition} is a list of the partitioned candidate features,
#' where each feature is represented by a vector of fundamental mutations
#' \item \code{select_n} is the number of top features to retain for each method
#' \item \code{mutation_dt} is a sorted data frame of candidate features 
#' and their AUC
#' \item \code{dt_new} is the transformed data from \code{transform_data}
#' }
#' 
#' @noRd
#' 
feature_selection <- function(dt,
                              factor,
                              wgs = FALSE){
        train <- dt
        n_iter <- 5
        n_fold <- 3
        inner_partitions <- vapply(seq_len(n_iter), 
                                   FUN = function(x){
                                     createFolds(y = train$IndVar,
                                                 k = n_fold)},
                                   FUN.VALUE = vector("list", length = n_fold))
        
        # Feature engineering
        new_partition <- feature_engineering(train, factor, inner_partitions, 
                                             n_fold, wgs)
        
        # Transform data
        dt_new <- transform_data(train, new_partition)
        
        # Trinucleotide ranking in inner folds
        rank_tri_ls <- rank_tri_features(dt_new, inner_partitions,
                                         new_partition, factor)
        
        # Get new partition after grouping trinucleotides of same rank
        repartition <- repartition_with_rank(rank_tri_ls)
        features_gmsa <- repartition[[1]]
        new_partition <- repartition[[2]]
        features_selected <- repartition[[3]]
        
        # Transform data 
        dt_new <- transform_data(dt, new_partition)
        train <- dt_new
        
        # Find n*
        n_star_ls <- n_star_features(train, factor, inner_partitions, 
                                     n_fold, features_selected)
        select_n <- median_n_star(n_star_ls) # take medians over inner folds
        select_n <- select_n_adjust(train, features_selected, select_n)
        
        # Return output
        assert_that(length(features_selected) >= 1, 
                    msg = "No predictive features found")
        
        out <- list(features_gmsa = features_gmsa, 
                    features_selected = features_selected,
                    new_partition = new_partition,
                    dt_new = dt_new,
                    select_n = select_n)
        return(out)
}

# simplify_signature.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Mar 31, 2021
#
# (Export) Function to simplify signature representation

# Convert single feature to simplified labels (helper function)
features_to_labels <- function(feature_name, features_partition){
    mutations <- features_partition[[feature_name]]
    label_mutations <- h_mix[vapply(h_mix, function(x){
        setequal(x, mutations)}, logical(1))]
    label <- names(label_mutations)
    if(length(label) == 1){
        return(list(label = label, multiplier = 1))
    } else {
        # If no exact match in h_mix, pick labels recursively to build feature
        label <- c()
        multiplier <- c()
        denominator <- sum(vapply(mutations, function(x){
            prop_muts_all[[x]][["TOTAL_MUTATIONS"]]},
            numeric(1)))
        while(length(mutations) > 0){
            # Pick h_mix mutation that explains as much as the feature
            # as possible
            candidate_labels <- h_mix[vapply(h_mix, function(x){
                length(setdiff(x, mutations)) == 0}, logical(1))]
            candidate_labels_lengths <- vapply(candidate_labels, 
                                               function(x) length(x),
                                               numeric(1))
            max_candidate <- vapply(candidate_labels, function(x){
                length(x) == max(candidate_labels_lengths)}, logical(1))
            label_mutations <- candidate_labels[max_candidate][1]
            label <- c(label, names(label_mutations))
            
            # Calculate the proportion of the feature that is
            # represented by the h_mix mutation 
            # (i.e. projection based on background ratio)
            numerator <- sum(vapply(label_mutations[[1]], function(x){
                prop_muts_all[[x]][["TOTAL_MUTATIONS"]]}, numeric(1)))
            multiplier <- c(multiplier, numerator/denominator) 
            mutations <- setdiff(mutations, label_mutations[[1]])
        }
    }
    return(list(label = label, multiplier = multiplier))
}

# Convert label to IUPAC naming
transform_to_iupac <- function(z){
    zp <- z
    for(i in seq_along(transform_iupac_grep)){
        zp <- vapply(zp, FUN = gsub, 
                     pattern = transform_iupac_grep[[i]],
                     replacement = names(transform_iupac_grep)[i],
                     character(1))
    }
    return(zp[[1]])
}

#' Function to simplify signature representation 
#' into interpretable labels for visualization purposes
#' 
#' Take a signature representation from SuperSig
#' and group trinucleotides within each feature into
#' interpretable labels, with optional IUPAC labeling
#' from IUPAC_CODE_MAP in the Biostrings package
#' 
#' @param object an object of class \code{SuperSig}
#' @param iupac logical value indicating whether to use IUPAC labels
#' (iupac = \code{TRUE}) or not (iupac = \code{FALSE}) 
#' 
#' @export
#' 
#' @return \code{simplify_signature} returns a vector of
#' simplified features and their difference in mean
#' mean rates between exposed and unexposed (or
#' average rate if the factor is "age")
#' 
#' @examples
#' 
#' head(example_dt) # use example data from package
#' input_dt <- make_matrix(example_dt) # convert to correct format
#' input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
#' supersig <- get_signature(data = input_dt, factor = "Smoking")
#' simplify_signature(object = supersig, iupac = FALSE)
#' simplify_signature(object = supersig, iupac = TRUE)
#' 
simplify_signature <- function(object, iupac){
    signature <- Signature(object)
    features_partition <- Features(object)
    signature_labeled = c()
    for(j in seq_along(signature)){
        feature_name <- names(signature[j])
        rate <- signature[[j]]
        labels_out <- features_to_labels(feature_name, features_partition)
        tmp_vec <- labels_out$multiplier*rate
        names(tmp_vec) <- labels_out$label
        signature_labeled <- c(signature_labeled, tmp_vec)
    }
    
    if(iupac){
        labels_iupac <- vapply(names(signature_labeled), 
                               function(x) transform_to_iupac(x),
                               character(1))
        names(signature_labeled) <- labels_iupac
    } 
    
    return(signature_labeled)
}

# my_auc.R
# -----------------------------------------------------------------------------
# Author: Bahman Afsari, Albert Kuo
# Date last modified: Feb 1, 2021
#
# Functions for calculating AUC using the Mann-Whitney U Statistic

#' Calculate AUC for a column
#' 
#' Calculate AUC for one column using the Mann-Whitney U statistic
#' 
#' @param y an indicator variable column, i.e. the "true value" column
#' @param x a column of numeric values, e.g. the projected counts of a feature
#' 
#' @return \code{my_auc} returns a numeric AUC value
#' 
#' @noRd
#' 
my_auc <- function(y, x){
    # Handle missing observations
    missing_x <- is.na(x)
    if(all(missing_x)){
        return(NA)
    } else if(any(missing_x)){
        x <- x[!missing_x]
        y <- y[!missing_x]
    }
    ind <- which(y == 1)
    n1 <- sum(y == 1)
    n0 <- sum(y == 0)
    diff_rank <- sum(rank(x, ties.method = "first")[ind]) - n1*(n1+1)/2 
    return(diff_rank/(n1*n0))                                        
}

y <- c(rep(0, 5), rep(1, 5))
x1 <- 1:10
x2 <- 10:1
x3 <- c(1, 1, 5, 7, 9, 2, 4, 6, 8, 10)
x4 <- c(1, 3, 5, 7, 9, 2, 4, 6, 8, 10)
x5 <- c(NA, 2:10)
x6 <- rep(NA, 10)
    
test_that("my_auc works", {
    expect_equal(my_auc(y, x1), 1)
    expect_equal(my_auc(y, x2), 0)
    expect_equal(my_auc(y, x3), 0.64)
    expect_equal(my_auc(y, x4), 0.6)
})

test_that("my_auc handles missing values", {
    expect_equal(my_auc(y, x5), 1)
    expect_equal(my_auc(y, x6), NA)
})

# transform_data.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Feb 15, 2021

# library(dplyr)
# library(here)

#' Transform data frame of mutations
#' 
#' Transform data frame of mutations by projecting counts for candidate features
#' 
#' @param dt a data frame of mutations
#' @param new_partition a partition of features from 
#' \code{generate_min_sigma_algebra}
#' 
#' @import dplyr
#' @importFrom rlang .data
#' 
#' @return \code{transform_data} returns a transformed data frame of mutations, 
#' with columns corresponding to the candidate features with projected counts
#' and other necessary columns (\code{IndVar}, \code{AGE}, 
#' \code{TOTAL_MUTATIONS}, and \code{DIVISON})
#' 
#' @noRd
#' 
transform_data <- function(dt, 
                           new_partition){
    # Add columns for every type of mutation
    dt <- dt %>% 
        mutate_(.dots = muts_formula) %>%
        mutate(tracking_ind = seq_len(nrow(dt)))
    
    # Store variables
    features_selected <- names(new_partition)
    
    # Count features by directly summing
    new_partition_formula <- vapply(new_partition, function(x) 
        paste0("(", paste("`", x, "`", sep = "", collapse = "+"), ")"),
        FUN.VALUE = character(1))
    
    dt_new <- dt %>% 
        mutate_(.dots = new_partition_formula)
    
    dt_new <- dt_new %>%
        arrange(.data$tracking_ind) %>%
        select(c(features_selected, "IndVar", "AGE", "TOTAL_MUTATIONS", 
                         matches("DIVISION"))) 
    
    return(dt_new)
}

# context_matters.R
# -----------------------------------------------------------------------------
# Author: Bahman Afsari, Albert Kuo
# Date last modified: Feb 18, 2021
#
# Function for binomial testing using a hierarchical tree structure

# Binomial test for all features in tree (helper function)
binom_test_tree <- function(tree){
    p_thresh <- 0.05
    bonf_correction <- 150
    
    tree <- tree %>%
        mutate(p_value = pbinom(.data$q, .data$size, .data$prob,
                                lower.tail = FALSE), 
               p_value = ifelse(.data$size == 0, 1, .data$p_value),
               p_value_bonf = .data$p_value*bonf_correction,
               sig = .data$p_value_bonf < p_thresh) 
    
    return(tree)
}

# Create tree (helper function)
create_tree <- function(muts_counts, background_probs){
    # Create "tree"
    tree <- tibble(feature = names(h_muts_index),
                   n_children = h_muts_index) %>%
        right_join(. , background_probs, by = "feature") %>%
        mutate(q = muts_counts[.data$feature],
               size = muts_counts[.data$parent_name],
               prop = .data$q/.data$size)
    
    # Binomial test for all features
    tree <- binom_test_tree(tree)
    
    # First level of significant features
    survival_mutations <- tree %>%
        filter(.data$n_children == 16) %>%
        filter(.data$sig) %>% 
        pull(.data$feature)
    
    # Keep tests only for significant parents
    for(i in c(4, 1)){
        tree_tmp <- tree %>%
            filter(.data$n_children == i) %>%
            mutate(sig_2 = ifelse(.data$parent_name %in% 
                                    c(survival_mutations, "TOTAL_MUTATIONS"), 
                                  .data$sig, TRUE))
        
        survival_mutations_tmp <- tree_tmp %>% 
            filter(.data$n_children == i) %>%
            group_by(.data$feature) %>%
            summarize(sig_2 = all(.data$sig_2)) %>%
            filter(.data$sig_2) %>%
            pull(.data$feature)
        
        survival_mutations <- c(survival_mutations, 
                                unique(survival_mutations_tmp))
    }
    
    survival_mutations_cache <- tree %>%
        filter(.data$feature %in% survival_mutations)
    
    return(list(tree, survival_mutations_cache, survival_mutations))
}

# Compute children of feature (helper function)
children_of_feature <- function(tree, survival_mutations, 
                                survival_children_level3, level){
    tree_pruned <- tree %>%
        filter(.data$feature %in% survival_mutations &
                 .data$n_children == ifelse(level == 1, 16, 4)) %>%
        select(.data$feature, .data$n_children, .data$parent_name, 
               .data$prob, .data$q, .data$size) %>%
        left_join(. , muts_children_level3_df, by = "feature") %>%
        inner_join(. , survival_children_level3, 
                   by = c("child_name", "parent_name" = "feature")) %>%
        filter(.data$parent_name %in% 
                 c(survival_mutations, "TOTAL_MUTATIONS")) %>%
        group_by(.data$feature, .data$parent_name, .data$prob, 
                 .data$q, .data$size) %>%
        summarize(prob_child = sum(.data$prob_child),
                  q_child = sum(.data$q_child)) %>%
        ungroup()
    
    return(tree_pruned)
}

# Compute children of parent (helper function)
children_of_parent <- function(tree_pruned, survival_children_level3){
    tree_pruned_parent <- tree_pruned %>%
        distinct(.data$parent_name) %>%
        rename(feature = .data$parent_name) %>%
        left_join(. , muts_children_level3_df, by = "feature") %>%
        inner_join(. , survival_children_level3,
                   by = c("child_name", "feature")) %>%
        group_by(.data$feature) %>%
        summarize(prob_parent_child = sum(.data$prob_child),
                  q_parent_child = sum(.data$q_child)) %>%
        ungroup()
    
    return(tree_pruned_parent)
}

# Join and test pruned tree (helper function)
test_pruned_tree <- function(tree_pruned, tree_pruned_parent){
    p_thresh <- 0.05
    bonf_correction <- 150
    
    tree_pruned <- tree_pruned %>%
        full_join(. , tree_pruned_parent,
                  by = c("parent_name" = "feature")) %>%
        mutate(prob_pruned = (.data$prob - .data$prob_child)/
                         (1 - .data$prob_parent_child),
               q_pruned = .data$q - .data$q_child,
               size_pruned = .data$size - .data$q_parent_child) %>%
        mutate(p_value = pbinom(.data$q_pruned, .data$size_pruned, 
                                .data$prob_pruned, lower.tail = FALSE), 
               p_value = ifelse(.data$size_pruned == 0, 1, .data$p_value),
               p_value_bonf = .data$p_value*bonf_correction,
               sig = .data$p_value_bonf < p_thresh)
    return(tree_pruned)
}


# Retest tree with significant children contributions removed (helper function)
retest_tree <- function(tree, survival_mutations_cache, survival_mutations){
    p_thresh <- 0.05
    bonf_correction <- 150
    
    for(level in 2:1){
        # Take union of children and grandchildren
        union_children_vec <- survival_mutations_cache %>%
            filter(.data$n_children %in% if(level == 1) c(4, 1) else c(1)) %>%
            left_join(. , muts_children_level3_df, by = "feature") %>%
            pull(.data$child_name) %>%
            unique()
        
        # All survival children and grandchildren as level 3 mutations
        survival_children_level3 <- tree %>%
            filter(.data$feature %in% union_children_vec) %>%
            select(.data$feature, .data$parent_name, .data$prob, .data$q) %>%
            rename(child_name = .data$feature,
                   feature = .data$parent_name,
                   prob_child = .data$prob,
                   q_child = .data$q)
        
        # Compute children of feature
        tree_pruned <- children_of_feature(tree, survival_mutations, 
                                           survival_children_level3, level)
        
        # Compute children of parent
        tree_pruned_parent <- children_of_parent(tree_pruned, 
                                                 survival_children_level3)

        # Join and test
        tree_pruned <- test_pruned_tree(tree_pruned, tree_pruned_parent)
        
        pruned_mutations <- tree_pruned %>%
            group_by(.data$feature) %>%
            summarize(sig = all(.data$sig)) %>%
            filter(!.data$sig) %>%
            pull(.data$feature)
        
        survival_mutations_cache <- survival_mutations_cache %>%
            filter(!(.data$feature %in% pruned_mutations))
        
        survival_mutations <- unique(survival_mutations_cache$feature)
    }
    
    return(survival_mutations)
}

#' Function for binomial testing using a hierarchical tree structure
#' 
#' Obtain a list of survival mutations by performing a series of nested 
#' binomial tests for selecting and pruning significant mutations
#' 
#' @param muts_df a data frame of mutations
#' @param tot_pseudo an optional numeric value for the pseudo-count value
#' (default is `0`)
#' @param wgs logical value indicating whether sequencing data is 
#' whole-genome (wgs = \code{TRUE}) or whole-exome (wgs = \code{FALSE}). 
#' 
#' @import dplyr
#' @import assertthat
#' @importFrom rlang .data
#' 
#' @return \code{context_matters} returns a vector of survival mutations that
#' pass the binomial tree testing
#' 
#' @noRd
#' 
context_matters <- function(muts_df, 
                            tot_pseudo = 0, 
                            wgs = FALSE){
    # Check input
    assert_that(all(setdiff(names(muts_df), "TOTAL_MUTATIONS") == 
                      names(muts_formula)),
                      msg = "Column names of input data are not correct 
                            in context_matters")
    
    # Use WGS as background probabilities
    if(wgs) background_probs <- background_probs_wgs
    
    muts_counts <- muts_df %>% colSums()
    # Pseudo-count
    for(feature in names(muts_counts)){
        all_possible_tri["TOTAL_MUTATIONS"] <- 3
        muts_counts[feature] <- muts_counts[feature] + 
            all_possible_tri[feature]*tot_pseudo/3
    }
    muts_counts <- vapply(muts_counts, round, FUN.VALUE = numeric(1))

    # Create tree
    tree_ls <- create_tree(muts_counts, background_probs)
    tree <- tree_ls[[1]]
    survival_mutations_cache <- tree_ls[[2]]
    survival_mutations <- tree_ls[[3]]
    
    # Retest tree after removing significant children
    survival_mutations <- retest_tree(tree, survival_mutations_cache, 
                                      survival_mutations)

    if(length(survival_mutations) == 0){
        return("TOTAL_MUTATIONS")
    } else {
        return(survival_mutations)
    }
}

#' Example dataset of mutations
#'
#' A dataset containing a list of mutations and other necessary attributes
#'
#' @format A data frame with 10 rows and 5 columns:
#' \describe{
#'   \item{sample_id}{ID of the patient}
#'   \item{age}{age of the patient}
#'   \item{chromosome}{chromosomal position of the mutation}
#'   \item{position}{position of the mutation}
#'   \item{ref}{original nucleotide}
#'   \item{alt}{mutated nucleotide}
#' }
"example_dt"

# convert_to_level3.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo
# Date last modified: Mar 1, 2021
#
# Helper function for GenerateMinSigmaAlgebra.R

#' Given a feature, convert it to a set of 
#' fundamental (level 3) mutation features
#' 
#' @param feature a feature (e.g. "C>T")
#' 
#' @return \code{convert_to_level3} returns a vector of 
#' fundamental (level 3) features
#' 
#' @noRd
#' 
convert_to_level3 = function(feature){
    out <- unname(muts_children_level3[[feature]])
    return(out)
}

# generate_min_sigma_algebra.R
# -----------------------------------------------------------------------------
# Author: Bahman Afsari, Albert Kuo
# Date last modified: Feb 15, 2021
#
# Function to create the smallest non-overlapping partition that generates 
# the minimal sigma algebra containing two sets of mutation features

#' Generate partition of features
#' 
#' Find the smallest non-overlapping partition that generates 
#' the minimal sigma algebra containing two sets of mutation features
#' 
#' @param input_ls a named list of length 2, where each element is a
#' vector of mutation features
#' @param condense an optional logical value indicating whether the output value
#' `new_partition` should return mutations in condensed form (default is 
#' \code{FALSE})
#' @param partitioned_features indicates whether the mutation features have 
#' been prepartitioned
#' (i.e. input is coming from output of generate_min_sigma_algebra)
#' 
#' @import dplyr
#' @import assertthat
#' 
#' @return \code{generate_min_sigma_algebra} returns a list of partitioned 
#' features, where each feature is represented by a vector of the fundamental 
#' (level 3) mutations
#' 
#' @noRd
#' 
generate_min_sigma_algebra <- function(input_ls, 
                                       condense = FALSE,
                                       partitioned_features = FALSE){
    # Check input
    assert_that(length(input_ls) == 2, msg = "input_ls length not equal to 2")
    assert_that(!is.null(names(input_ls)), msg = "input_ls is not named")
    
    if(partitioned_features){
        feat_ls <- unlist(input_ls, recursive = FALSE)
    } else {
        # Convert to fundamental (i.e. level 3) mutations
        feat_ls <- unique(unlist(input_ls))
        feat_ls <- lapply(feat_ls, convert_to_level3)
    }
    
    # Create matrix of indicator values
    temp <- setNames(rep(FALSE, length(muts_level3)), muts_level3)
    M <- vapply(feat_ls, FUN = function(mutation)
        {out <- temp; out[mutation] <- TRUE; out},
        FUN.VALUE = vector("logical", length = length(muts_level3)))
    
    # Separate mutations that do not appear in any of feat_ls
    any_ind <- apply(M, MARGIN = 1, FUN = any)
    other_muts <- rownames(M)[!any_ind] 
    M <- M[any_ind, ]
    
    # Create partition
    if(is.null(dim(M))){ # If there is only one feature
        new_partition <- list(names(M))
    } else {
        new_partition <- lapply(rownames(M), FUN = function(mutation){
            apply(cbind(M[, which(M[mutation, ] == TRUE)],
                        !M[, which(M[mutation, ] == FALSE)]),
                        MARGIN = 1, FUN = all)} %>% 
                which() %>% names())
        new_partition <- new_partition[!duplicated(new_partition)]
    }
    names(new_partition) <- paste0("F", seq_along(new_partition))
    
    # Add back other mutations as "Remaining" feature
    if(length(other_muts) > 0)
        new_partition[["Remaining"]] <- other_muts 
    
    # Condense mutations
    if(condense)
        new_partition <- lapply(new_partition, condense_mutations) 
    
    # Return output
    return(new_partition)
}

x1 <- list(A = c("A[C>A]T"))
x2 <- list(c("A[C>A]T"), c("A[C>A]T"))
x3 <- list(A = c("A[C>A]T"), B = c("A[C>A]T"))

test_that("gmsa valid input", {
    expect_error(generate_min_sigma_algebra(x1),
                 "input_ls length not equal to 2")
    expect_error(generate_min_sigma_algebra(x2), 
                 "input_ls is not named")
})

input_ls = list(Exposed = c("[C>A]C", "T[T>G]T"), 
                Unexposed = c("C>A", "T[T>G]T"))
ref <- list(F1 = c("[C>A]A", "[C>A]G", "[C>A]T"),
            F2 = c("[C>A]C"),
            F3 = c("T[T>G]T"),
            Remaining = c("C>G", "C>T", "T>A", "T>C", "[T>G]A",
                          "[T>G]C", "[T>G]G", "A[T>G]T", "C[T>G]T",
                          "G[T>G]T"))
test_that("gmsa works", {
    out <- generate_min_sigma_algebra(input_ls, condense = TRUE)
    expect_equal(out, ref)
})

# get_signature.R
# -----------------------------------------------------------------------------
# Author: Albert Kuo, Yifan Zhang
# Date last modified: Mar 12, 2019
#
# (Export) Function to calculate signature

#' Function to obtain a SuperSig
#' 
#' Generate a tissue-specific SuperSig for a given dataset of mutations and
#' exposure factor. Returns the SuperSig and a classification model trained 
#' with the SuperSig.
#' 
#' @param data a data frame of mutations containing columns for
#' \code{sample_id}, \code{age}, \code{IndVar}, and the 96 trinucleotide
#' mutations (see vignette for details)
#' @param factor the factor/exposure (e.g. "age", "smoking"). If the 
#' factor = "age", the SuperSig is computed using
#' counts. Otherwise, rates (counts/age) are used.
#' @param wgs logical value indicating whether sequencing data is 
#' whole-genome (wgs = \code{TRUE}) or whole-exome (wgs = \code{FALSE}) 
#' 
#' @import dplyr
#' 
#' @export
#' 
#' @return \code{get_signature} returns an object of class \code{SuperSig}
#' 
#' @examples
#' 
#' head(example_dt) # use example data from package
#' input_dt <- make_matrix(example_dt) # convert to correct format
#' input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
#' get_signature(data = input_dt, factor = "Age") # get SuperSig
#' 
get_signature <- function(data, factor, wgs = FALSE){
    # Capitalize factor string
    factor <- toupper(factor)
    
    # Check column names of data
    if(is.na(match("SAMPLE_ID",toupper(colnames(data))))){
        stop('Input data frame missing sample_id column.')
    } else {
        colnames(data)[match("SAMPLE_ID",toupper(colnames(data)))]<-"sample_id"
    }
    if(is.na(match("AGE",toupper(colnames(data))))){
        stop('Input data frame missing AGE column.')
    } else {
        colnames(data)[match("AGE",toupper(colnames(data)))] <- "AGE"
    }
    if(is.na(match("INDVAR",toupper(colnames(data))))){
        stop('Input data frame missing IndVar column.')
    } else {
        colnames(data)[match("INDVAR",toupper(colnames(data)))] <- "IndVar"
    }

    if(nrow(data) < 5) stop('More than 5 samples required for get_signature.')
    
    # Check all trinucleotides are present
    trinucleotideBases <- unique(transform_muts_vec)
    if(all(trinucleotideBases %in% toupper(colnames(data)))){
        data$TOTAL_MUTATIONS <- rowSums(data[,trinucleotideBases])
    } else {
        stop('Input data frame missing one or more trinucleotide mutations.')
    }
    
    # Get features
    features_out <- suppressWarnings(feature_selection(data, factor, wgs))
    
    # Get apparent AUC and model
    classification_out <- supersig_classifier(dt = features_out$dt_new,
                                              factor = factor,
                                              keep_classifier = TRUE,
                                              features_selected = 
                                                features_out$features_selected,
                                              select_n = features_out$select_n)
    
    # Create and return S4 object
    feature_names <- names(classification_out$signature$mean_diffs)
    out <- SuperSig(Signature = classification_out$signature$mean_diffs,
                    Features = features_out$new_partition[feature_names],
                    AUC = classification_out$auc,
                    Model = classification_out$classifier)
    return(out)
}

x1 <- tibble(x = 1,
             age = 2,
             indvar = FALSE)
x2 <- tibble(sample_id = 1,
             x = 2,
             indvar = FALSE)
x3 <- tibble(sample_id = 1,
             age = 2,
             x = FALSE)

test_that("get_signature valid input", {
    expect_error(get_signature(x1, factor = "f"),
                 "Input data frame missing sample_id column")
    expect_error(get_signature(x2, factor = "f"),
                 "Input data frame missing AGE column")
    expect_error(get_signature(x3, factor = "f"),
                 "Input data frame missing IndVar column")
    
})

# all_my_auc.R
# -----------------------------------------------------------------------------
# Author: Bahman Afsari, Albert Kuo
# Date last modified: Dec 10, 2019

#' Calculate AUC for every column
#' 
#' all_my_auc calls my_auc to calculate the AUC for every column other than the 
#' indicator variable
#' 
#' @param z a matrix
#' @param IndVar indicator variable for the exposure
#'
#' @return \code{all_my_auc} returns a list of AUCs, 
#' one for every column (feature)
#' 
#' @noRd
#' 
all_my_auc <- function(z, IndVar){
    cols <- setdiff(colnames(z), IndVar)
    apply(z[, cols], MARGIN = 2, FUN = my_auc, y = z %>% pull(!!IndVar))
}

---
title: "Using supersigs"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using supersigs}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  warning = FALSE,
  comment = "#>"
)
```

```{r setup}
library(supersigs)
```

# Introduction

The `supersigs` package implements the supervised method proposed by
*Afsari, et al.* to find signatures ("SuperSigs"). In this vignette, 
we cover how to preprocess your data and run the method in `supersigs`.

# Preprocessing your data

## VCF file

If you have a VCF file, you can use `readVcf` from the `VariantAnnotation` 
package to read in your VCF file as a VCF object. The age of each patient 
should be stored as `age` in the `colData` of your `VCF` object. Then use 
`process_vcf` to transform the VCF object into a simplified data frame 
format, which will be explained further in [Example data].

If you do not have a VCF file, skip to [Example data].

```{r}
# Load packages for make_matrix function
suppressPackageStartupMessages({
  library(VariantAnnotation)
})

fl <- system.file("extdata", "chr22.vcf.gz", package="VariantAnnotation")
vcf <- VariantAnnotation::readVcf(fl, "hg19") 
# Subset to first sample
vcf <- vcf[, 1]
# Subset to row positions with homozygous or heterozygous alt
positions <- geno(vcf)$GT != "0|0" 
vcf <- vcf[positions[, 1],]
colData(vcf)$age <- 50    # Add patient age to colData
dt <- process_vcf(vcf)
head(dt)
```

## Example data

The method uses single-base mutations in exomic data from cancer samples.
Specifically, it requires data on every sample's mutations, the positions
of those mutations, and the age of all patients. This data can be represented
as a list of mutations. Below is an example dataset (stored and accessible 
from the `supersigs` R package). If you have a VCF file, read the [VCF file]
section to see how to process your data into the following format. 

* `sample_id` is an ID for each sample
* `age` is the age of the patient
* `chromosome` and `position` is the position of the mutation
* `ref` is the original nucleotide
* `alt` is the mutated nucleotide 

```{r}
head(example_dt)
```

## Transform data

Once you've read in your data, you will need to transform it into a data 
frame of features before running the core functions. This involves 
2 steps:

1. First, we assume that mutations are the same regardless of the strand on 
which it occurred. For example, this means that C>A mutations are considered 
the same as G>T mutations and we will convert all G>T mutations to be denoted 
as C>A mutations.

2. Because the features used are built upon trinucleotide features 
(e.g. A[C>A]T), this will require matching your mutations to a reference genome
to identify what the flanking bases of every mutation are. In our example 
below, we will use the `hg19` reference genome.

Both of these steps are done by the `make_matrix` function. Note that using
the `make_matrix` function requires installing and loading a reference 
genome (`BSgenome.Hsapiens.UCSC.hg19` and `BSgenome.Hsapiens.UCSC.hg38` 
are supported).

```{r}
# Load packages for make_matrix function
suppressPackageStartupMessages({
  library(BSgenome.Hsapiens.UCSC.hg19)
})
```

We apply `make_matrix` to transform our example dataset (`example_dt`) into
a data frame of trinucleotide mutations (`input_dt`), which is the format 
required by the `supersigs` R package. Each row in `input_dt` corresponds to
a different patient and the values in the columns are the number of mutations
for each trinucleotide mutation.

```{r}
input_dt <- make_matrix(example_dt)
head(input_dt)
```

# Getting your signature

To apply the supervised method on your data, run the `get_signature` function.
The function has two parameters: an input data frame `data` and the `factor` 
(e.g. `factor = "Smoking"`). `data` is a data frame with the following columns:

* `IndVar` (indicator variable) is a logical indicator for whether they were
exposed to the `factor` or not
* `sample_id` is an ID for each sample
* `age` is the age of the patient
* columns of counts for all 96 trinucleotide mutations

The process of converting a VCF file to this format is covered in 
[Preprocessing your data]. An example for `data` is printed below.

```{r}
suppressPackageStartupMessages({
  library(dplyr)
})

# Add IndVar column
input_dt <- input_dt %>%
  mutate(IndVar = c(1, 1, 1, 0, 0)) %>%
  relocate(IndVar)

head(input_dt)
```

Once you have the correct data format, apply `get_signature` to the 
dataset to get your `SuperSig`, which is an S4 object containing four slots:

* `Signature` is the signature, represented as their differences in mean 
rates (or the overall mean rate if the factor is "age") between the two groups
(exposed versus unexposed)
* `Features` is the list of features that comprise the signature and their 
representation in terms of the fundamental (trinucleotide) mutations
* `AUC` is the apparent AUC of the model (i.e. not cross-validated)
* `Model` is the list containing the trained logistic regression model 
(glm class)

```{r}
set.seed(1)
supersig <- get_signature(data = input_dt, factor = "Smoking")
supersig
```

To obtain a signature representation that is more interpretable, 
you can group the trinucleotide features within each feature 
using the `simplify_signature` function (with an option to use
[IUPAC](https://www.bioinformatics.org/sms/iupac.html) labels). 
This is useful for making plots of signatures.

```{r}
features <- simplify_signature(object = supersig, iupac = FALSE)
features_iupac <- simplify_signature(object = supersig, iupac = TRUE)
```

```{r}
library(ggplot2)
data.frame(features = names(features_iupac),
           differences = features_iupac) %>%
  ggplot(aes(x = features, y = differences)) +
  geom_col() +
  theme_minimal()
```

# Using a signature

To apply the `SuperSig` to a new dataset, use the `predict_signature` function.
This function returns the new dataset with columns for feature counts for the 
signature and a score column for the predicted classification score.

Below is an example for the `SuperSig` we trained in the previous section. We 
reuse `input_dt` as our "new data" for illustrative purposes, but in 
practice, you would use a different dataset from the one that was used to 
train the signature (e.g. a test set).

```{r}
newdata = predict_signature(supersig, newdata = input_dt, factor = "Smoking")

newdata %>%
  select(X1, score)
```

In addition, you may wish to use a `SuperSig` pre-trained on TCGA data. 
These are accessible from the package in `supersig_ls`, where each element
of the list is a `SuperSig`. There are 67 SuperSigs that have been trained 
on various tissues and factors. The names are printed below (formatted as 
"factor (tissue)"). Details regarding the training of these signatures are 
discussed in *Afsari, et al. (2021, ELife)*.

```{r}
names(supersig_ls)
```

```{r}
# Use pre-trained signature
newdata = predict_signature(supersig_ls[["SMOKING (LUAD)"]], 
                            newdata = input_dt, factor = "Smoking")
newdata %>%
  select(IndVar, X1, X2, X3, score)
```


# Partially supervised signatures

In some cases, you may be interested in removing the contribution of a 
supervised signature from your data frame of mutations as a way to adjust 
for a particular factor. For example, suppose that we are interested in the
deciphering a signature for smoking in lung cancer. We can first remove the 
contribution of the aging signature in lung cancer, before learning the 
smoking signature with a supervised or unsupervised method.
We discuss in *Afsari, et al. (2021, ELife)* how doing so can lead to 
better performance.

```{r}
adjusted_dt <- partial_signature(data = input_dt, object = supersig)
head(adjusted_dt)
```

# Session info

```{r sessionInfo}
sessionInfo()
```

```{r eval = F, include=F}
build_vignettes()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/supersig_ls.R
\docType{data}
\name{supersig_ls}
\alias{supersig_ls}
\title{Trained SuperSigs from TCGA}
\format{
A named list with 67 elements, each of which is a `SuperSig`
}
\usage{
supersig_ls
}
\description{
A list containing 67 SuperSigs
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SuperSig-class.R
\docType{class}
\name{SuperSig-class}
\alias{SuperSig-class}
\alias{SuperSig}
\title{An S4 class for SuperSig}
\description{
An S4 class for SuperSig
}
\section{Slots}{

\describe{
\item{\code{Signature}}{data frame of features and their 
difference in mean rates between exposed and unexposed
(or the average rate if the factor is "age")}

\item{\code{Features}}{list of features that comprise the signature and 
their representation in terms of the fundamental (trinucleotide) mutations}

\item{\code{AUC}}{length-one numeric vector of the apparent AUC 
(i.e. not cross-validated)}

\item{\code{Model}}{list of a glm class for trained logistic regression model}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simplify_signature.R
\name{simplify_signature}
\alias{simplify_signature}
\title{Function to simplify signature representation 
into interpretable labels for visualization purposes}
\usage{
simplify_signature(object, iupac)
}
\arguments{
\item{object}{an object of class \code{SuperSig}}

\item{iupac}{logical value indicating whether to use IUPAC labels
(iupac = \code{TRUE}) or not (iupac = \code{FALSE})}
}
\value{
\code{simplify_signature} returns a vector of
simplified features and their difference in mean
mean rates between exposed and unexposed (or
average rate if the factor is "age")
}
\description{
Take a signature representation from SuperSig
and group trinucleotides within each feature into
interpretable labels, with optional IUPAC labeling
from IUPAC_CODE_MAP in the Biostrings package
}
\examples{

head(example_dt) # use example data from package
input_dt <- make_matrix(example_dt) # convert to correct format
input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
supersig <- get_signature(data = input_dt, factor = "Smoking")
simplify_signature(object = supersig, iupac = FALSE)
simplify_signature(object = supersig, iupac = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict_signature.R
\name{predict_signature}
\alias{predict_signature}
\title{Function to predict using SuperSig object}
\usage{
predict_signature(object, newdata, factor)
}
\arguments{
\item{object}{an object of class \code{SuperSig}}

\item{newdata}{a data frame of mutations containing columns for
\code{sample_id}, \code{age}, \code{IndVar}, and the 96 trinucleotide
mutations (see vignette for details)}

\item{factor}{the factor/exposure (e.g. "age", "smoking")}
}
\value{
\code{predict_signature} returns the original data frame with 
additional columns for the feature counts and classification score
}
\description{
Using a generated SuperSig, predict on a new dataset and return 
predicted probabilities for each observation.
}
\examples{

head(example_dt) # use example data from package
input_dt <- make_matrix(example_dt) # convert to correct format
input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
out <- get_signature(data = input_dt, factor = "Age") # get SuperSig
newdata <- predict_signature(out, newdata = input_dt, factor = "age")
suppressPackageStartupMessages({library(dplyr)})
head(newdata \%>\% select(score))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_signature.R
\name{get_signature}
\alias{get_signature}
\title{Function to obtain a SuperSig}
\usage{
get_signature(data, factor, wgs = FALSE)
}
\arguments{
\item{data}{a data frame of mutations containing columns for
\code{sample_id}, \code{age}, \code{IndVar}, and the 96 trinucleotide
mutations (see vignette for details)}

\item{factor}{the factor/exposure (e.g. "age", "smoking"). If the 
factor = "age", the SuperSig is computed using
counts. Otherwise, rates (counts/age) are used.}

\item{wgs}{logical value indicating whether sequencing data is 
whole-genome (wgs = \code{TRUE}) or whole-exome (wgs = \code{FALSE})}
}
\value{
\code{get_signature} returns an object of class \code{SuperSig}
}
\description{
Generate a tissue-specific SuperSig for a given dataset of mutations and
exposure factor. Returns the SuperSig and a classification model trained 
with the SuperSig.
}
\examples{

head(example_dt) # use example data from package
input_dt <- make_matrix(example_dt) # convert to correct format
input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
get_signature(data = input_dt, factor = "Age") # get SuperSig

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_matrix.R
\name{make_matrix}
\alias{make_matrix}
\title{Function to transform mutations into "matrix" format}
\usage{
make_matrix(data, genome = "hg19")
}
\arguments{
\item{data}{a data frame of mutations in VCF format 
(see vignette for details)}

\item{genome}{the reference genome used ("hg19" or "hg38")}
}
\value{
\code{make_matrix} returns a data frame of mutations,
one row per sample
}
\description{
Transform a data frame of mutations in long format into 
a data frame of trinucleotide mutations with flanking bases
in a wide matrix format.
}
\examples{

head(example_dt) # use example data from package
input_dt <- make_matrix(example_dt) # convert to correct format
head(input_dt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_vcf.R
\name{process_vcf}
\alias{process_vcf}
\title{Function to transform VCF object into "matrix" format}
\usage{
process_vcf(vcf)
}
\arguments{
\item{vcf}{a VCF object (from \code{VariantAnnotation} package)}
}
\value{
\code{process_vcf} returns a data frame of mutations, 
one row per mutation
}
\description{
Transform a VCF object into 
a data frame of trinucleotide mutations with flanking bases
in a wide matrix format. The function assumes that the VCF object
contains only one sample and that each row in rowRanges 
represents an observed mutation in the sample.
}
\examples{

# Use example vcf from VariantAnnotation
suppressPackageStartupMessages({library(VariantAnnotation)})
fl <- system.file("extdata", "chr22.vcf.gz", package="VariantAnnotation")
vcf <- VariantAnnotation::readVcf(fl, "hg19") 

# Subset to first sample
vcf <- vcf[, 1]
# Subset to row positions with homozygous or heterozygous alt
positions <- geno(vcf)$GT != "0|0" 
vcf <- vcf[positions[, 1],]
colData(vcf)$age <- 50        # Add patient age to colData (optional)

# Run function
dt <- process_vcf(vcf)
head(dt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_dt.R
\docType{data}
\name{example_dt}
\alias{example_dt}
\title{Example dataset of mutations}
\format{
A data frame with 10 rows and 5 columns:
\describe{
  \item{sample_id}{ID of the patient}
  \item{age}{age of the patient}
  \item{chromosome}{chromosomal position of the mutation}
  \item{position}{position of the mutation}
  \item{ref}{original nucleotide}
  \item{alt}{mutated nucleotide}
}
}
\usage{
example_dt
}
\description{
A dataset containing a list of mutations and other necessary attributes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partial_signature.R
\name{partial_signature}
\alias{partial_signature}
\title{Function to remove the contribution of a SuperSig}
\usage{
partial_signature(data, object)
}
\arguments{
\item{data}{a data frame of mutations containing columns for
\code{sample_id}, \code{age}, \code{IndVar}, and the 96 trinucleotide
mutations (see vignette for details)}

\item{object}{an object of class \code{SuperSig}}
}
\value{
\code{predict_signature} returns the original data frame with 
the contribution of a supervised signature removed
}
\description{
Remove the contribution of a SuperSig from the data and return the data.
}
\examples{

head(example_dt) # use example data from package
input_dt <- make_matrix(example_dt) # convert to correct format
input_dt$IndVar <- c(1, 1, 1, 0, 0) # add IndVar column
supersig <- get_signature(data = input_dt, factor = "Age") # get SuperSig
partial_signature(data = input_dt, object = supersig)

}
